<?php



require_once 'Classes/Model/CalibrationHeader.php';
require_once 'Classes/Model/ObservationHeader.php';
require_once 'Classes/Model/ObservationSet.php';
require_once 'Classes/General/Pair.php';



/**
 * @class CalUpdater
 *
 * This algorithm takes an array of calibration set objects and a range
 * to operate on, and gets all the calibrations for those observations
 * within the cal window specified
 **/
class CalUpdater
{
  // PUBLIC
  
  public function __construct()
  {
    $this->observation_sets = null;
    $this->cal_window = 0;
    $this->range = null;
    $this->database = null;
  }
  
  public function SetRange( Pair $s_range ) { $this->range = $s_range; }
  
  public function SetObservationSets( &$s_observation_sets ) { $this->observation_sets = $s_observation_sets; }
  
  public function SetCalsWindow( $s_cal_window ) { $this->cal_window = $s_cal_window; }
  
  public function SetDatabase( $s_database ) { $this->database = $s_database; }
  
  public function Execute()
  {
    if( $this->observation_sets == null )
      throw new GZ_Exception( 'observation sets not set', 'CalUpdater::Execute' );
    
    if( $this->range == null )
      throw new GZ_Exception( 'range not set', 'CalUpdater::Execute' );
    
    if( $this->cal_window == 0 )
      throw new GZ_Exception( 'cal window is zero ', 'CalUpdater::Execute' );
    
    if( $this->database == null )
      throw new GZ_Exception( 'database not set', 'CalUpdater::Execute' );
    
    if( count( $this->observation_sets ) == 0 )
      throw new GZ_Exception( 'observation sets is empty', 'CalUpdater::Execute' );
    
    if( $this->range->second > count( $this->observation_sets ) )
      throw new GZ_Exception( 'range goes beyond observation set boundaries', 'CalUpdate::Execute' );
    
    for( $ind = $this->range->first; $ind < $this->range->second; $ind ++ )
    {
      $next_observation_set = $this->observation_sets[$ind];
      if( $next_observation_set->GetCalibrationData() == null )
      {
	$next_mjd = $next_observation_set->GetObservationHeader()->GetMJD();
	$cal_query = new CalQuery();
	$cal_query->SetDatabase( $this->database );
	$half_minute = 0.5 / (24.0 * 60.0);
	$start_time = $next_mjd->ToString() - ( $this->cal_window * $half_minute);
	$end_time = $next_mjd->ToString() + ( $this->cal_window * $half_minute);
	$cal_query->SetStartTime( MJD::ConvertString( $start_time ) );
	$cal_query->SetEndTime( MJD::ConvertString( $end_time ) );
	$cal_query->Execute();
	$next_calibrations = $cal_query->GetCalibrations();
	$this->observation_sets[$ind]->SetCalibrations( $next_calibrations );
      }
    }
  }
  
  // PRIVATE

  private $database;
  private $range;
  private $cal_window;
  private $observation_sets;
};

?>