<?php



require_once 'Classes/General/File.php';
require_once 'Classes/Model/CalibrationHeader.php';
require_once 'Classes/Interfaces/CalInterface.php';



class ObservationHeader
{
  public function __construct( ObservationHeader $rhs = null )
  {
    $this->file_id = null;
    $this->coordinates = null;
    $this->distance = null;
    $this->pulsar_name = null;
    $this->mjd = null;
    $this->data_type = null;
    $this->date = null;
    $this->file = null;
    $this->ut = null;
    $this->bw = null;
    $this->obs_freq = null;
    $this->scan_len = null;
    $this->nbin = null;
    $this->npol = null;
    $this->nchan = null;
    $this->nsub = null;
    $this->band_prefix = '';

    if( $rhs != null )
      $this->Copy( $rhs );
  }

  public function Copy( ObservationHeader $rhs )
  {
    $this->file_id = $rhs->file_id;
    $this->coordinates = new SkyCoord( $rhs->coordinates );
    $this->distance = $rhs->distance;
    $this->pulsar_name = $rhs->pulsar_name;
    $this->mjd = new MJD( $rhs->mjd );
    $this->data_type = $rhs->data_type;
    $this->date = new Date( $rhs->date );
    $this->file = new File($rhs->file);
    $this->ut = $rhs->ut;
    $this->bw = $rhs->bw;
    $this->obs_freq = $rhs->obs_freq;
    $this->scan_len = $rhs->scan_len;
    $this->nbin = $rhs->nbin;
    $this->npol = $rhs->npol;
    $this->nchan = $rhs->nchan;
    $this->nsub = $rhs->nsub;
    $this->band_prefix = $rhs->band_prefix;
  }

  /**
   * This function extracts the parameters for this object from the 
   * database record given.
   **/

  public function FromRecord( $record )
  {
    $this->SetPulsarName( $record['src_name'] );
    $this->SetFileID( $record['file_id'] );
    $this->SetDistance( $record['distance'] );
    
    $this->SetCoordinates( SkyCoord::ConvertRADEC( Angle::ConvertDegrees( $record['rajd'] ),
			                           Angle::ConvertDegrees( $record['decjd'] ) ) );
    
    $mjd = new MJD();
    $mjd->FromString( $record['MJD'] );
    $this->SetMJD( $mjd );
    
    $this->SetBW( $record['bw'] );
    
    $this->SetObsFreq( $record['obsfreq'] );
    
    $this->SetScanLen( $record['scanlen'] );
    
    $this->SetProjectID( $record['project_id'] );
    
    $this->file = new File();
    $this->file->SetPath( $record['data_loc'] );
    $this->file->SetFilename( $record['filename'] );
    
    $date = new Date();
    $date->FromDMYString( $record['date'] );
    $this->SetDate( $date );
    
    $this->SetUT( $record['ut'] );
    
    $this->SetDataType( $record['data_type'] );
    
    $this->SetNBin( $record['nbin'] );
    $this->SetNPol( $record['npol'] );
    $this->SetNChan( $record['nchan'] );
    $this->SetNSub( $record['nsub'] );
    
    $this->SetBandPrefix( $record['prefix'] );
  }

  public function GetFileID() { return $this->file_id; }
  public function SetFileID( $s_file_id ) { $this->file_id = $s_file_id; }
  
  public function GetPulsarName() { return $this->pulsar_name; }
  public function SetPulsarName( $s_pulsar_name ) { $this->pulsar_name = $s_pulsar_name; }
  
  public function GetDistance() { return $this->distance; }
  public function SetDistance( $s_distance ) { $this->distance = $s_distance; }
  
  public function GetMJD() { return $this->mjd; }
  public function SetMJD( $s_mjd ) { $this->mjd = $s_mjd; }
  
  public function GetProjectID() { return $this->project_id; }
  public function SetProjectID( $s_project_id ) { $this->project_id = $s_project_id; }
  
  public function GetDataType() { return $this->data_type; }
  public function SetDataType( $s_data_type ) { $this->data_type = $s_data_type; }
  
  public function GetFile() { return $this->file; }
  public function SetFile( $s_file ) { $this->file = $s_file; }
  
  public function GetDate() { return $this->date; }
  public function SetDate( $s_date ) { $this->date = $s_date; }
  
  public function GetUT() { return $this->ut; }
  public function SetUT( $s_ut ) { $this->ut = $s_ut; }
  
  public function SetBW( $s_bw ) { $this->bw = $s_bw; }
  public function GetBW() { return $this->bw; }
  
  public function SetObsFreq( $s_obs_freq ){ $this->obs_freq = $s_obs_freq; }
  public function GetObsFreq() { return $this->obs_freq; }
  
  public function SetScanLen( $s_scan_len ) { $this->scan_len = $s_scan_len; }
  public function GetScanLen() { return $this->scan_len; }
  
  public function GetCoordinates() { return $this->coordinates; }
  public function SetCoordinates( $s_coordinates ) { $this->coordinates = $s_coordinates; }
  
  public function GetNPol() { return $this->npol; }
  public function SetNPol( $s_npol ) { $this->npol = $s_npol; }
  
  public function GetNChan() { return $this->nchan; }
  public function SetNChan( $s_nchan ) { $this->nchan = $s_nchan; }
  
  public function GetNSub() { return $this->nsub; }
  public function SetNSub( $s_nsub ) { $this->nsub = $s_nsub; }
  
  public function GetNBin() { return $this->nbin; }
  public function SetNBin( $s_nbin ) { $this->nbin = $s_nbin; }
  
  public function GetBandPrefix() { return $this->band_prefix; }
  public function SetBandPrefix( $s_band_prefix ) { $this->band_prefix = $s_band_prefix; }
  
  
  // PRIVATE

  private $file_id;
  private $pulsar_name;
  private $mjd;
  private $project_id;
  private $file;
  private $data_type;
  private $ut;
  private $date;
  private $bw;
  private $obs_freq;
  private $scan_len;
  private $coordinates;
  private $npol;
  private $nbin;
  private $nchan;
  private $nsub;
  private $band_prefix;

  // STATIC

  public static $valid_data_types = Array( 'DFB', 'WBC', 'CPSR2', 'FB_1BIT' );
}



?>