<?php



require_once 'Classes/Model/ObservationHeader.php';
require_once 'Classes/Pulsar/ObservationData2.php';



/**
 * @class ObservationSet
 *
 * This class represents an observation and a set of associated calibrations. These
 * calibrations will be within X minutes of the observation
 **/
class ObservationSet
{
  // PUBLIC

  public function __construct( ObservationSet $rhs = null )
  {
    $this->observation_data = null;
    $this->calibrations = Array();
    $this->data_dir = Dir::ConvertPathString( '/pulsar/scratch01/data_archive/' );

    if( $rhs != null )
      $this->Copy( $rhs );
  }
  public function Copy( ObservationSet $rhs )
  {
    $this->observation_data = new ObservationData2( $rhs->observation_data );

    $this->calibrations = Array();
    foreach( $rhs->calibrations as $next_cal )
      $this->calibrations[] = new ObservationData2( $next_cal );
  }

  // MAIN INTERFACE
  
  public function SetObservationHeader( ObservationHeader $s_observation ) { $this->priv_SetObservationHeader( $s_observation ); }
  public function GetObservationData() { return $this->observation_data; }
  
  public function GetObservationHeader() { return $this->observation_data->GetObservationHeader(); }

  public function SetCalibrations( Array $s_calibrations ) { $this->priv_SetCalibrations( $s_calibrations ); }
  public function GetCalibrationData() { return $this->calibrations; }
  
  public function SetScrunchingParameters( ScrunchingParameters $s_scr_params ) 
                  { $this->priv_SetScrunchingParameters( $s_scr_params ); }
  
  public function EstimateDownloadableSize() { return $this->priv_EstimateDownloadableSize(); }
  
  public function GetSourceFile() { $this->priv_GetSourcefile(); }
  
  public function SetDataDir( $s_data_dir ) { $this->data_dir = $s_data_dir; }
  
  public function GetDownloadableObsFile() { return $this->priv_GetDownloadableObsFile(); }
  public function GetDownloadableCalFiles() { return $this->priv_GetDownlodableCalFiles(); }
  
  // PRIVATE

  private $observation_data;
  private $calibrations;
  private $data_dir;
  
  private function priv_SetScrunchingParameters( $s_scr_params )
  {
    $this->observation_data->SetScrunchingParameters( $s_scr_params );
  }
  
  private function priv_EstimateDownloadableSize()
  {
    $dl_size = $this->observation_data->EstimateDownloadableSize();
    foreach( $this->calibrations as $next_cal )
      $dl_size += $next_cal->EstimateDownloadableSize();
    
    return $dl_size;
  }
  
  private function priv_GetDownloadableObsFile()
  {
    return $this->observation_data->MakeDownloadableFile();
  }
  
  private function priv_GetDownloadableCalFiles()
  {
    $cal_files = Array();
    
    foreach( $this->calibrations as $next_cal_data )
    {
      $cal_files[] = $next_cal_data->MakeDownloadableFile();
    }
    
    return $cal_files;
  }

  private function priv_SetObservationHeader( ObservationHeader $obs_header )
  {
    $this->observation_data = new ObservationData2();
    $this->observation_data->SetObservationHeader( $obs_header );
    $this->observation_data->SetDataDir( Dir::ConvertPathString(DATA_DIR) );
    if( $this->scrunching_parameters != null )
        $this->observation_data->SetScrunchingParameters( $this->scrunching_parameters );
  }
  
  private function priv_SetCalibrations( Array $cal_headers )
  {
    foreach( $cal_headers as $next_header )
    {
      $new_data = new ObservationData2();
      $new_data->SetDataDir( DIR::ConvertPathString(DATA_DIR) );
      $new_data->SetObservationHeader( $next_header );
      if( $this->scrunching_parameters != null )
          $new_data->SetScrunchingParameters( $this->scrunching_parameters );
      $this->calibrations[] = $new_data;
    }
  }
};

