<?php



require_once 'Classes/Model/ObservationHeader.php';
require_once 'Classes/Pulsar/ArchiveScruncher.php';
require_once 'Classes/Pulsar/ScrunchingParameters.php';
require_once 'Classes/Pulsar/CPSR2Combiner.php';



class ObservationData2
{
  // PUBLIC

  public function __construct( ObservationData2 $src = null )
  {
    if( $src != null )
      $this->Copy( $src );
    else
    {
      $this->observation_header = null;
      $this->data_file = null;
      $this->scrunched_file = null;
      $this->downloadable_file = null;
      $this->scrunching_parameters = null;
      $this->data_dir = null;
      $this->web_dir = null;
    }
  }
  
  public function Copy( ObservationData2 $src )
  {
    $this->observation_header = new ObservationHeader( $src->observation_header );
    $this->data_file = new File( $src->data_file );
    $this->scrunched_file = new File( $src->scrunched_file );
    $this->downloadable_file = new File( $src->downloadable_file );
    $this->scrunching_parameters = null;
    if( $src->scrunching_parameters != null )
      $this->scrunching_parameters = new ScrunchingParameters( $src->scrunching_parameters );
    $this->data_dir = new Dir( $src->data_dir );
    $this->web_dr = new Dir( $src->web_dir );
  }
  
  // MAIN INTERFACE
  
  public function GetOriginalFile() { return $this->priv_GetOriginalFile(); }
  public function GetDataFile() { return $this->priv_GetDataFile(); }
  public function GetScrunchedFile() { return $this->priv_GetScrunchedFile(); }
  public function GetDownloadableFile() { return $this->priv_GetDownloadableFile(); }
  
  public function MakeDataFile() { return $this->priv_MakeDataFile(); }
  public function MakeScrunchedFile() { return $this->priv_MakeScrunchedFile(); }
  public function MakeDownloadableFile() { return $this->priv_MakeDownloableFile(); }
  
  public function SetObservationHeader( ObservationHeader $s_observation_header ) { $this->observation_header = $s_observation_header; }
  public function GetObservationHeader() { return $this->observation_header; }
  
  public function SetScrunchingParameters( ScrunchingParameters $s_params ) { $this->scrunching_parameters = $s_params; }
  public function GetScrunchingParameters() { return $this->scrunching_parameters; }
  
  public function SetDataDir( Dir $s_data_dir ) { $this->data_dir = $s_data_dir; }
  public function SetWebDir( Dir $s_web_dir ) { $this->web_dir = $s_web_dir; }
  
  public function EstimateDownloadableSize() { return $this->priv_EstimateDownloadableSize(); }
  
  // PRIVATE

  private $observation_header;
  private $data_file;
  private $scrunched_file;
  private $downloadable_file;
  private $scrunching_parameters;
  private $data_dir;
  
  
  /// This function returns a File object pointing to the original data file or directory.
  /// The original file always exists.
  
  private function priv_GetOriginalFile()
  {
    if( $this->observation_header == null )
      throw new GZ_Exception( 'Observation header not set', 'ObservationData::priv_GetOriginalFile' );
    
    $this->original_file = $this->observation_header->GetFile();
    
    return $this->original_file;
  }
  
  /// This function returns a File object showing where the data file will be
  /// If the file does not yet exist, it will NOT be created.
  
  private function priv_GetDataFile()
  {
    if( $this->data_file == null )
    {
      if( $this->observation_header == null )
        throw new GZ_Exception( "Observation header not set", "ObservationData2::priv_GetDataFile" );
      
      if( $this->data_dir == null )
        throw new GZ_Exception( "The data directory is not set", "ObservationData2::priv_GetDataFile" );
      
      if( !$this->data_dir->Exists() )
        throw new GZ_Exception( "Data directory given does not exist", "ObservationData2::priv_GetDataFile" );
      
      if( $this->original_file == null )
        $this->GetOriginalFile();
      
      $this->data_file = new File();
      $this->data_file->SetPath( $this->data_dir->GetPath() );
      
      $data_filename = $this->original_file->GetFilename();
      $band_prefix = $this->observation_header->GetBandPrefix();
      if( $band_prefix != "\0" )
        $data_filename .= $band_prefix;
      
      $this->data_file->SetFilename( $data_filename );
    }
    
    return $this->data_file;
  }
  

  /// This function returns a File object showing where the scrunched file will be
  /// If the actual file doesn't yet exist on disk it will not create it, or any
  /// of the intermediate files.
  
  public function priv_GetScrunchedFile()
  {
    if( $this->scrunching_parameters == null )
      throw new GZ_Exception( 'Scrunching parameters not set', 'ObservationData2::priv_GetScrunchedFile' );
    
    if( !$this->scrunching_parameters->ScrunchingSet() )
      throw new GZ_Exception( 'Scrunching parameters set but no scrunching required', 'ObservationData2::priv_GetScrunchedFile' );
    
    if( $this->data_file == null )
        $this->GetDataFile();
    
    $scr_filename = AddPreExtension( $this->data_file->GetFilename(), $this->scrunching_parameters->GetTextDescription() );
    
    $this->scrunched_file = File::ConvertFullPath( $this->data_dir->GetPath() . '/' . $scr_filename );
    
    return $this->scrunched_file;
  }
  
  /// This function figures out where the downloadable file will be, it will not create the file
  /// or any of the files prerequisits, it just determines the path and filename.
  
  private function priv_GetDownloadableFile()
  {
    if( $this->scrunching_parameters != null &&
        $this->scrunching_parameters->ScrunchingSet() )
      $this->downloadable_file = $this->GetScrunchedFile();
    else
      $this->downloadable_file = $this->GetDataFile();
    
    return $this->downloadable_file;
  }
  
  /// This function returns a File object pointing to the data file, if the file 
  /// does not yet exist, it will create it.
  
  private function priv_MakeDataFile()
  {
    if( $this->data_file == null )
      $this->GetDataFile();
    
    if( !$this->data_file->Exists() )
    {
      if( $this->original_file == null )
        $this->GetOriginalFile();
      
      if( $this->original_file->IsDir() )
      {
        $src_dir = Dir::ConvertFile( $this->original_file );
        $dir_contents = $src_dir->ListFiles();
  
        $combiner = new CPSR2Combiner();
        foreach( $dir_contents as $next_file )
        {
          $prefix = $this->observation_header->GetBandPrefix();
          if( substr( $next_file, 0, 1 ) == $prefix && 
              ( strpos( $next_file, '.cfb') !== false ||
                strpos( $next_file, '.cf' ) !== false ) )
          {
            $combiner->AddArchive( File::ConvertFullPath( $src_dir->GetPath() . '/' . $next_file ) );
          }
        }
        $combiner->SetDestFile( $this->data_file );
        $combiner->Execute();
      }
      else
      {
        symlink( $original_file->GetFullPath(), $this->data_file->GetFullPath() );
      }
    }
    
    return $this->data_file;
  }
  
  /// This function returns a File object for the scrunched file, if the file
  /// does not yet exist, it will create it.
  
  private function priv_MakeScrunchedFile()
  {
    if( $this->scrunching_parameters == null )
      throw new GZ_Exception( "Scrunching parameters don't exist", "ObservationData2::MakeScrunchedFile" );
    
    if( !$this->scrunching_parameters->ScrunchingSet() )
      throw new GZ_Exception( "Scrunching parameters exist, but not set", "ObservationData2::MakeScrunchedFile" );
    
    if( $this->scrunched_file == null )
      $this->GetScrunchedFile();
    
    if( !$this->scrunched_file->Exists() )
    {
      if( !$this->data_file->Exists() )
        $this->MakeDataFile();
      
      $scruncher = new ArchiveScruncher();
      $scruncher->SetParameters( $this->scrunching_parameters );
      $scruncher->SetSrcFile( $this->data_file );
      $scruncher->SetDestFile( $this->scrunched_file );
      $scruncher->Execute();
    }
    
    return $this->scrunched_file;
  }
  
  /// This function returns a File object for the downloadable file, if the file
  /// does not yet exist, it will create it.
  
  private function priv_MakeDownloableFile()
  {
    if( $this->downloadable_file == null )
      $this->GetDownloadableFile();
    
    if( !$this->downloadable_file->Exists() )
    {
      if( $this->scrunching_parameters != null )
      {
        $this->MakeScrunchedFile();
        $this->downloadable_file = $this->scrunched_file;
      }
      else
      {
        $this->MakeDataFile();
        $this->downloadable_file = $this->data_file;
      }
    }
    
    return $this->downloadable_file;
  }
  
  /// This function uses whatever files are currently available to make an 
  /// estimate of the final file size, without creating any new files.
  /// TODO rewrite this to take advantage of any exsting files.
  
  private function priv_EstimateDownloadableSize()
  {
    $nbin = $this->observation_header->GetNBin();
    $nchan = $this->observation_header->GetNChan();
    $nsub = $this->observation_header->GetNSub();
    $npol = $this->observation_header->GetNPol();
    
    $data_size = 2 * $nbin * $nchan * $nsub * $npol;
    
    if( $this->scrunching_parameters != null )
    {
      $scr_nchan = $this->scrunching_parameters->GetFScrunch();
      $scr_nsub = $this->scrunching_parameters->GetTScrunch();
      $scr_npol = $this->scrunching_parameters->GetPScrunch();
      
      if( $scr_nchan == null )
        $scr_nchan = 1;
      if( $scr_nsub == null )
        $scr_nsub = 1;
      if( $scr_npol == null )
        $scr_npol = false;

      $new_nchan = ceil( $nchan / $scr_nchan );
      $new_nsub = ceil( $nsub / $scr_nsub );
      $new_npol = $npol;
      if( $scr_npol )
        $new_npol = 1;
      
      $new_data_size = 2 * $nbin * $new_nchan * $new_nsub * $new_npol;
      
      $data_size = $new_data_size;
    }
    
    return $data_size;
  }
};

